 /* -*- Mode: c -*-

     Lexer for a minimal programming language

     Copyright 2016 by Stephan Schulz, schulz@eprover.org.

     This code is released under the GNU General Public Licence
     Version 2.
 */

%option noyywrap
%option yylineno

%{
#include <string.h>

enum yytokentype
{
   OPENPAR,
   CLOSEPAR,
   MULT,
   DIV,
   PLUS,
   MINUS,
   EQ,
   NEQ,
   LT,
   GT,
   LEQ,
   GEQ,
   OPENCURLY,
   CLOSECURLY,
   SEMICOLON,
   COMMA,
   IDENT,
   STRINGLIT,
   INTLIT,
   INTEGER,
   STRING,
   IF,
   WHILE,
   RETURN,
   PRINT,
   ELSE,
   ERROR
};


char* tokname[] = 
{
   "OPENPAR",
   "CLOSEPAR",
   "MULT",
   "DIV",
   "PLUS",
   "MINUS",
   "EQ",
   "NEQ",
   "LT",
   "GT",
   "LEQ",
   "GEQ",
   "OPENCURLY",
   "CLOSECURLY",
   "SEMICOLON",
   "COMMA",
   "IDENT",
   "STRINGLIT",
   "INTLIT",
   "INTEGER",
   "STRING",
   "IF",
   "WHILE",
   "RETURN",
   "PRINT",
   "ELSE",
   "ERROR"
};


int yycol = 0;

void print_tok(int tok)
{
   printf("<%5ul:%3lu: %s[%ul]=\"%s\">\n", yylineno, yycol-yyleng, tokname[tok], tok, yytext);
}

#define YY_USER_ACTION {yycol += yyleng;}


%}

DIGIT      [0-9]
ALPHA      [a-zA-Z]
ALNUM      {ALPHA}|{DIGIT}

INT        {DIGIT}+
ID         {ALPHA}{ALNUM}*


%%

"("  {print_tok(OPENPAR);}
")"  {print_tok(CLOSEPAR);}
"*"  {print_tok(MULT);}
"/"  {print_tok(DIV);}
"+"  {print_tok(PLUS);}
"-"  {print_tok(MINUS);}
"="  {print_tok(EQ);}
"!=" {print_tok(NEQ);}
">"  {print_tok(GT);}
"<"  {print_tok(LT);}
">=" {print_tok(GEQ);}
"<=" {print_tok(LEQ);}
"{"  {print_tok(OPENCURLY);}
"}"  {print_tok(CLOSECURLY);}
";"  {print_tok(SEMICOLON);}
","  {print_tok(COMMA);}


Integer {print_tok(INTEGER);}
String  {print_tok(STRING);}
if      {print_tok(IF);}
while   {print_tok(WHILE);}
return  {print_tok(RETURN);}
print   {print_tok(PRINT);}
else  {print_tok(ELSE);}
       
{ALPHA}{ALNUM}* {print_tok(IDENT);}

\"[^"]*\"       {print_tok(STRINGLIT);}

{DIGIT}+        {print_tok(INTLIT);}


#[^\n]*\n { /* Eat comments */}


[\n]    {yycol = 0;}

[\r\t ] { /* Skip  whitespace*/ }


 /* Everything else is an invalid character. */
.   {print_tok(ERROR);}

%%



int main( int argc, char **argv )       
{
   ++argv, --argc;  /* skip over program name */
   if ( argc > 0 )
      yyin = fopen( argv[0], "r" );
   else
      yyin = stdin;
   
   yylex();
}
